/* 
 *
 *    error_ctx.h
 *
 *    $Source: /usr/local/CVS/error_ctx/src/error_ctx.h,v 
 *    $Revision: 1.13 
 *    $Date: 1997/06/06 00:39:13 
 *    $Name: SETREF_1_0_Release 
 *    $Disclaimer: 
 *    ----------------------------------------------------------------------
 *    SET Draft Reference Implementation Disclaimer/License
 *    June 30, 1997
 *    
 *    This is a beta version of the SET Draft Reference Implementation
 *    ("DRI").  It is a work in progress and is being provided at this
 *    stage and in this form to help developers and other interested
 *    parties to understand the SET protocol.  This version of the DRI
 *    provides the core functionality required to process a simple
 *    transaction.  This release does not support certificate authority
 *    messages. Nor does it support the SET option to perform
 *    transactions without cardholder certificates.  For more details
 *    on the functionality in this beta version, please refer to the
 *    attached documentation.  
 *    
 *    Later versions of the DRI will include additional functionality
 *    and may include other changes.  DO NOT RELY UPON THIS CODE IN YOUR
 *    DEVELOPMENT.  If you discover a bug, please let us know by posting
 *    a message to the SET-DEV mailing list.  Our lawyers made us include
 *    the next four paragraphs, so PLEASE READ THE TEXT BELOW because it
 *    describes what you can do with the DRI.
 *     
 *    Copyright (C) 1996, 1997, Visa International Service Association and
 *    MasterCard International Incorporated.  All rights reserved.  
 *    Permission to copy and implement the material contained herein is
 *    granted subject to the conditions that (i) any copy or re-publication
 *    must bear this legend in full; (ii) any derivative work must bear
 *    a notice that it is not the Draft Reference Implementation ("DRI")
 *    for SET and (iii) that none of the copyright holders shall have any
 *    responsibility or liability whatsoever to any other party arising
 *    from the use or publication of the material contained herein.
 *    
 *    Visa and MasterCard ("Licensors") hereby grant you ("Licensee") a
 *    limited license to (i) make, use, copy and modify the DRI for the
 *    purpose of developing software that complies with the SET
 *    Specification ("SET Products") and for the purpose of achieving a
 *    better understanding of the SET Protocol; and (ii) distribute SET
 *    Products that incorporate all or a portion of the DRI and other
 *    software, which together are provided under a defined interface
 *    different from that provided by the DRI's API.  Licensee shall not
 *    redistribute or sell a license to the DRI nor shall Licensee
 *    incorporate any part of the DRI into a SET Toolkit.  A "Toolkit"
 *    means a product that provides comprehensive facilities to enable
 *    other parties to create software products for distribution. 
 *    Bolt-on applications and payment modules shall not be considered
 *    Toolkits for the purpose of this license.  The DRI is not intended
 *    to be used as a Toolkit.
 *    
 *    Licensors make no representation or warranty of any kind, express
 *    or implied, relating to the DRI.  Licensors are making the DRI
 *    publicly available as an aid to understanding, creating, and
 *    testing SET Products.  Licensees are solely responsible for any
 *    use of the DRI and agree to hold Licensors harmless for any claim
 *    that may arise out of any product created with the DRI.  
 *    
 *    Licensee expressly acknowledges Terisa Systems, Inc., a Delaware
 *    Corporation with offices at 4984 El Camino Real, Los Altos,
 *    California, as a third party beneficiary of this license.
 *    
 *    This software is controlled material under U.S. law and the laws
 *    of other jurisdictions and you hereby acknowledge and agree that
 *    you must fully comply with all the current laws of the United States
 *    including, without limitation, rules and regulations of the United
 *    States Office of Export Administration and other applicable U.S.
 *    governmental agencies.  
 *    ----------------------------------------------------------------------
 *     $
 *
 *    Copyright (C) 1996, 1997, Visa International Service Association and MasterCard International Incorporated
 *    All Rights Reserved.
 *
 *    briank@terisa.com
 *
 */



/**
 *    @header       error_ctx.h
 *    @one-liner    standard error-reporting mechanism
 *    @comment      <error_ctx> is an abstraction used for reporting
 *                  errors.  The mechanism is used to convert
 *                  return values to textual error descriptions,
 *                  to log these descriptions, and to provide global
 *                  contextual error state in a multithreaded environment.
 *    @comment      Two macros, <ERROR_MODULE()> and <ERROR_NUMBER()>
 *                  convert a numeric return value to values representing
 *                  the module that generated the error and the
 *                  module-specific error number. 
 *    @comment      In the event that the default <error_ctx>,
 *                  <DEFAULT_ERROR_CTX>, is not used, each thread is
 *                  expected to use a unique <error_ctx>. 
 *    @comment      The error handling routines <error_explain_error()>,
 *                  <error_log_error()>, and <error_reset_error()> always
 *                  return <NO_ERROR>, independent of their error status,
 *                  in order to prevent errors from generating additional
 *                  errors.
 *    @domain       public
 *    @example
 *
 *    logging_example()
 *    {
 *        int             status;         // numeric return value 
 *        int             module;         // the module in which the error occured 
 *
 *        status = some_function();
 *        if (status != NO_ERROR) {
 *
 *            module = ERROR_MODULE(status);
 *
 *            // print arbitrary data to the log 
 *            error_log_printf(DEFAULT_ERROR_CTX, module, "Something bad happened\n");
 *
 *            // log the error 
 *            error_log_error(DEFAULT_ERROR_CTX, status);
 *        }
 *
 *        // ... 
 *    }
 *
 *    @example
 *
 *    explain_example()
 *    {
 *        int             status;         // numeric return value 
 *        UCHAR           str[ERROR_MESSAGE_LENGTH];
 *
 *        status = some_function();
 *        if (status != NO_ERROR) {
 *
 *            // obtain an explaination of the error 
 *            error_explain_error(DEFAULT_ERROR_CTX, status, str);
 *
 *            // display the explanation 
 *            printf("error message: %s\n", str);
 *        }
 *
 *        // ... 
 *    }
 *
 *    @example
 *
 *    int
 *    write_log(write_handle, buf, nbyte)
 *        VOID     *write_handle;
 *        char     *buf;
 *        int       nbyte;
 *    {
 *        return fwrite(buf, 1, nbyte, (FILE*)write_handle);
 *    }
 *    multithreaded_example()
 *    {
 *        error_ctx       ectx;           // per-thread error_ctx 
 *        int             status;         // numeric return value 
 *        FILE           *log;            // thread-specific error log 
 *
 *        log = fopen(thread_specific_error_log(), "a");
 *        assert(log != 0);
 *
 *        status = create_error_ctx(&ectx, write_log, (VOID*)log);
 *        assert(status == NO_ERROR);
 *
 *        status = some_function();
 *        if (status != NO_ERROR) {
 *            // log the error 
 *            error_log_error(ectx, status);
 *
 *            // display error information to the user 
 *            fprintf(stderr,"error number %d in module %d\n",
 *                    ERROR_NUMER(status), ERROR_MODULE(status));
 *        }
 *
 *        // ... 
 *    }
 *
 *    @bugs         
 *    @see          
 */


#ifndef __ERROR_CTX_H__
#define __ERROR_CTX_H__


#include <define.h>


#ifdef __cplusplus
extern "C" {
#endif


/**
 *    @typedef      error_ctx
 *    @one-liner    <error_ctx> type definition 
 *    @comment
 *    @domain       public
 *    @prototype    typedef VOID  *error_ctx;
 *    @example
 *    @bugs
 *    @see          
 */

typedef VOID  *error_ctx;



/* error_ctx==0 causes results in default error explanation */
/* str==0 causes error_explain_error to do nothing */
PUBLIC int    error_explain_error
  PROTO_LIST((
    error_ctx     ctx,                          /* IN, may be 0 */
    int           err,                          /* IN */
    char          str[ERROR_MESSAGE_LENGTH]     /* OUT, may be 0 */
  ));

/* error_ctx==0 causes error_log_error to do nothing */
PUBLIC int    error_log_error
  PROTO_LIST((
    error_ctx     ctx,                          /* IN, may be 0 */
    int           err                           /* IN */
  ));

/* error_ctx==0 causes error_log_printf to do nothing */
PUBLIC int    error_log_printf
  PROTO_LIST((
    error_ctx     ctx,                          /* IN, may be 0 */
    int           module_num,                   /* IN */
    char         *format,                       /* IN, may be 0 */
    ...
  ));

/* error_ctx==0 causes error_reset_error to do nothing */
PUBLIC int    error_reset_error
  PROTO_LIST((
    error_ctx     ctx,                          /* IN, may be 0 */
    int           err                           /* IN */
  ));






/* call to globally disable/enable logging */
PUBLIC int    error_disable_error_logging  PROTO_LIST((void));

PUBLIC int    error_enable_error_logging   PROTO_LIST((void));


/* call with ERROR_MODULE(error_num) to disable/enable logging for a
 * specific module */
PUBLIC int    error_disable_module_error_logging
  PROTO_LIST((
    int           module_num                    /* IN */
  ));

PUBLIC int    error_enable_module_error_logging
  PROTO_LIST((
    int           module_num                    /* IN */
  ));





/* write_func ==0 prevents logging */
/* write_func() returns the number of bytes written, or -1 for error */
PUBLIC int    create_error_ctx
  PROTO_LIST((
    error_ctx    *ctx,                          /* OUT */
    int         (*write_func) PROTO_LIST((VOID *write_handle, char *buf, int nbyte)),
                                                /* IN, may be 0 */
    VOID         *write_handle                  /* IN */
  ));

PUBLIC int    delete_error_ctx
  PROTO_LIST((
    error_ctx    *ctx                           /* IN/OUT */
  ));

PUBLIC int    modify_error_ctx
  PROTO_LIST((
    error_ctx     ctx,                          /* IN */
    int           modify,                       /* IN */
    ...                                         /* IN */
  ));

PUBLIC int    query_error_ctx
  PROTO_LIST((
    error_ctx     ctx,                          /* IN */
    int           query,                        /* IN */
    ...                                         /* IN */
  ));

/* the following default is the default on a per-error_ctx basis */
#define ERROR_CTX_DEFAULT_ALL_LOGGING_DISABLED       0




/* 
 *    DEFAULT_ERROR_CTX
 */

#define DEFAULT_ERROR_CTX        get_default_error_ctx()

PUBLIC error_ctx    get_default_error_ctx PROTO_LIST((void));

PUBLIC int          set_default_error_ctx PROTO_LIST((error_ctx ctx));







#ifdef __cplusplus
}
#endif
#endif

